
/*]  Copyright (c) 2009-2010, Charles McGarvey  [**************************
**]  All rights reserved.
*
* vi:ts=4 sw=4 tw=75
*
* Distributable under the terms and conditions of the 2-clause BSD license;
* see the file COPYING for a complete text of the license.
*
**************************************************************************/

#ifndef _MAIN_HH_
#define _MAIN_HH_

/**
 * \file Main.hh
 * This is where all the fun begins.
 */

#include <iostream>
#include <string>

#include <boost/shared_ptr.hpp>

#include <moof/dispatcher.hh>
#include <moof/math.hh>
#include <moof/view.hh>


namespace moof
{
	class settings;
	class view;
}


class Main;
typedef boost::shared_ptr<Main> MainP;

class Main : public moof::view
{
public:

	Main(moof::settings& settings, moof::video& video);

	void update(moof::scalar t, moof::scalar dt);
	void draw(moof::scalar alpha) const;
	bool handle_event(const moof::event& event);

	static std::string getSearchPath();
	static std::string getConfigPath();

	static void printUsage();
	static void printInfo(int argc, char* argv[]);

private:

	/**
	 * Set OpenGL to a state we can know and depend on.
	 */
	static void setupGL();

	moof::dispatcher::handle mNewContextDispatch;
};


#endif // _MAIN_HH_

